{% extends "core_main_app/_render/admin/theme/tools/section.html" %}

{% block section_title %}PID Settings{% endblock %}

{% block section_content %}
    <div class="alert alert-danger">
        <i class="fas fa-exclamation-triangle"></i>
        {{ data.error }}
    </div>
{% endblock %}